"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
function getNodes(node) {
    var states = node.states;
    var nodes = Object.keys(states).reduce(function (accNodes, stateKey) {
        var subState = states[stateKey];
        var subNodes = getNodes(states[stateKey]);
        accNodes.push.apply(accNodes, [subState].concat(subNodes));
        return accNodes;
    }, []);
    return nodes;
}
exports.getNodes = getNodes;
function getEdges(node, visited) {
    if (visited === void 0) { visited = {}; }
    var states = node.states;
    visited[node.key] = true;
    var subNodeEdges = Object.keys(states).reduce(function (_edges, key) {
        if (visited[key]) {
            return _edges;
        }
        var subState = states[key];
        _edges.push.apply(_edges, getEdges(subState, visited));
        visited[key] = true;
        return _edges;
    }, []);
    if (!node.on) {
        return subNodeEdges;
    }
    var edges = Object.keys(node.on).reduce(function (accEdges, action) {
        if (!node.on || !node.parent) {
            return accEdges;
        }
        var subStateKey = node.on[action];
        var subNode = node.parent.getState(subStateKey);
        var edge = { action: action, source: node, target: subNode };
        accEdges.push(edge);
        if (!visited[subStateKey]) {
            accEdges.push.apply(accEdges, getEdges(subNode, visited));
            visited[subStateKey] = true;
        }
        return accEdges;
    }, []);
    return subNodeEdges.concat(edges);
}
exports.getEdges = getEdges;
function getAdjacencyMap(node) {
    var actionMap = node.parent
        ? {}
        : undefined;
    var adjacency = {};
    if (node.on) {
        for (var _i = 0, _a = Object.keys(node.on); _i < _a.length; _i++) {
            var action = _a[_i];
            if (!node.parent || !actionMap) {
                continue;
            }
            var nextState = node.parent.getState(node.on[action]);
            var nextStateId = nextState.id;
            if (nextState.initial) {
                nextStateId += "." + nextState.initial;
            }
            actionMap[action] = nextStateId;
        }
    }
    if (actionMap) {
        adjacency[node.id] = actionMap;
    }
    if (node.states) {
        for (var _b = 0, _c = Object.keys(node.states); _b < _c.length; _b++) {
            var stateKey = _c[_b];
            var state = node.states[stateKey];
            var stateAdjacency = utils_1.mapValues(getAdjacencyMap(state), function (value) {
                return __assign({}, actionMap, value);
            });
            Object.assign(adjacency, stateAdjacency);
        }
    }
    return adjacency;
}
exports.getAdjacencyMap = getAdjacencyMap;
function getShortestPaths(machine) {
    if (!machine.states || !machine.initial) {
        return undefined;
    }
    var adjacency = getAdjacencyMap(machine);
    var initialId = machine.states[machine.initial].id;
    var pathMap = (_a = {},
        _a[initialId] = [],
        _a);
    shortestPaths(adjacency, initialId, pathMap);
    return pathMap;
    var _a;
}
exports.getShortestPaths = getShortestPaths;
function shortestPaths(adjacency, stateId, pathMap, visited) {
    if (visited === void 0) { visited = new Set(); }
    visited.add(stateId);
    var actionMap = adjacency[stateId];
    for (var _i = 0, _a = Object.keys(actionMap); _i < _a.length; _i++) {
        var action = _a[_i];
        var nextStateId = actionMap[action];
        if (!pathMap[nextStateId] ||
            pathMap[nextStateId].length > pathMap[stateId].length + 1) {
            pathMap[nextStateId] = pathMap[stateId].concat([stateId]);
        }
    }
    for (var _b = 0, _c = Object.keys(actionMap); _b < _c.length; _b++) {
        var action = _c[_b];
        var nextStateId = actionMap[action];
        if (visited.has(nextStateId)) {
            continue;
        }
        shortestPaths(adjacency, nextStateId, pathMap, visited);
    }
    return pathMap;
}
exports.shortestPaths = shortestPaths;
